//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


// SearchListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "SearchListCtrl.h"
#include "otherfunctions.h"


// CSearchListCtrl

IMPLEMENT_DYNAMIC(CSearchListCtrl, CListCtrl)
CSearchListCtrl::CSearchListCtrl(){
	memset(&asc_sort,0,6);
}

void CSearchListCtrl::Init(CSearchList* in_searchlist){
	SetExtendedStyle(LVS_EX_FULLROWSELECT);
	searchlist = in_searchlist;
	InsertColumn(0,GetResString(IDS_SW_SEARCHBOX) + " / " + GetResString(IDS_SW_NAME),LVCFMT_LEFT,100,0);
	InsertColumn(1,GetResString(IDS_DL_FILENAME),LVCFMT_LEFT,250,1);
	InsertColumn(2,GetResString(IDS_DL_SIZE),LVCFMT_LEFT,100,2);
	InsertColumn(3,GetResString(IDS_DL_SOURCES),LVCFMT_LEFT,50,3);
	InsertColumn(4,GetResString(IDS_AVAIL),LVCFMT_LEFT,70,4);
	InsertColumn(5,GetResString(IDS_TYPE),LVCFMT_LEFT,65,5);
	InsertColumn(6,GetResString(IDS_FILEID),LVCFMT_LEFT,220,6);

	m_SearchFileMenu.CreatePopupMenu();
	m_SearchFileMenu.AddMenuTitle(GetResString(IDS_FILE));
	m_SearchFileMenu.AppendMenu(MF_STRING,MP_RESUME, GetResString(IDS_DOWNLOAD));
	m_SearchFileMenu.AppendMenu(MF_STRING,MP_GETED2KLINK, GetResString(IDS_DL_LINK1));
	m_SearchFileMenu.AppendMenu(MF_STRING,MP_GETHTMLED2KLINK, GetResString(IDS_DL_LINK2));
	m_SearchFileMenu.AppendMenu(MF_SEPARATOR);
	m_SearchFileMenu.AppendMenu(MF_STRING,MP_REMOVESELECTED, GetResString(IDS_REMOVESELECTED));
	m_SearchFileMenu.AppendMenu(MF_STRING,MP_REMOVE, GetResString(IDS_REMOVESEARCHSTRING));
	m_SearchFileMenu.AppendMenu(MF_STRING,MP_REMOVEALL, GetResString(IDS_REMOVEALLSEARCH));
	m_SearchFileMenu.SetDefaultItem(MP_RESUME);
}

CSearchListCtrl::~CSearchListCtrl(){
}

BEGIN_MESSAGE_MAP(CSearchListCtrl, CListCtrl)
	ON_NOTIFY_REFLECT ( NM_CUSTOMDRAW, OnCustomdraw )
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnClick)
	ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()

// CSearchListCtrl message handlers

void CSearchListCtrl::AddResult(CSearchFile* toshow){
	uint32 itemnr = GetItemCount();
	InsertItem(LVIF_TEXT|LVIF_PARAM,itemnr,toshow->GetSearchString(),0,0,1,(LPARAM)toshow);
	SetItemText(itemnr,1,toshow->GetFileName());
	uint32 filesize = toshow->GetIntTagValue(FT_FILESIZE); 
	char buffer[50];
	CastItoXBytes(filesize,buffer);
	SetItemText(itemnr,2,buffer);
	itoa(toshow->GetIntTagValue(FT_SOURCES),buffer,10);
	SetItemText(itemnr,3,buffer);
	SetItemText(itemnr,5,GetFiletypeByName(toshow->GetFileName()));
	buffer[0] = 0;
	for (uint16 i = 0;i != 16;i++) // hmm I wonder if there is a standart function for this
		sprintf(buffer,"%s%02X",buffer,toshow->GetFileHash()[i]);
	SetItemText(itemnr,6,buffer);
}

void CSearchListCtrl::UpdateSources(CSearchFile* toupdate){
	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)toupdate;
	uint16 index = FindItem(&find);
	char buffer[50];
	itoa(toupdate->GetSourceCount(),buffer,10);
	SetItemText(index,3,buffer);
	SetItemText(index,0,toupdate->GetSearchString());
}

void CSearchListCtrl::RemoveResult(CSearchFile* toremove){
	LVFINDINFO find;
	find.flags = LVFI_PARAM;
	find.lParam = (LPARAM)toremove;
	sint32 result = FindItem(&find);
	if(result != (-1) )
		this->DeleteItem(result);
}

void CSearchListCtrl::OnCustomdraw(NMHDR* pNMHDR, LRESULT* pResult ){
NMLVCUSTOMDRAW* pLVCD = reinterpret_cast<NMLVCUSTOMDRAW*>( pNMHDR );

    *pResult = CDRF_DODEFAULT;

    if (CDDS_PREPAINT == pLVCD->nmcd.dwDrawStage){
        *pResult = CDRF_NOTIFYITEMDRAW;
    }
    else if (CDDS_ITEMPREPAINT == pLVCD->nmcd.dwDrawStage){
		uint32 blue = 0;

		CSearchFile* file = (CSearchFile*)pLVCD->nmcd.lItemlParam;
if (!file) return;

		blue = (file->GetSourceCount()-1) * 22;
		if (blue > 255) blue = 255;

		pLVCD->clrText = RGB(0,0,blue);
        *pResult = CDRF_DODEFAULT;
     }
}

void CSearchListCtrl::OnColumnClick( NMHDR* pNMHDR, LRESULT* pResult){
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// if it's a second click on the same column then reverse the sort order,
	// otherwise sort the new column in ascending order.
	asc_sort[pNMListView->iSubItem] = !asc_sort[pNMListView->iSubItem];
	SortItems(&SortProc,pNMListView->iSubItem+ ((asc_sort[pNMListView->iSubItem])? 0:10));
	*pResult = 0;
}

int CSearchListCtrl::SortProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort){
	CSearchFile* item1 = (CSearchFile*)lParam1;
	CSearchFile* item2 = (CSearchFile*)lParam2;	
	switch(lParamSort){
		case 0:
			return item1->GetSearchString().Compare(item2->GetSearchString());
		case 10:
			return item2->GetSearchString().Compare(item1->GetSearchString());
		case 1: //filename asc
			return strcmpi(item1->GetFileName(),item2->GetFileName());
		case 11: //filename desc
			return strcmpi(item2->GetFileName(),item1->GetFileName());
		case 2: //size asc
			return item2->GetIntTagValue(FT_FILESIZE) - item1->GetIntTagValue(FT_FILESIZE);
		case 12: //size desc
			return item1->GetIntTagValue(FT_FILESIZE) - item2->GetIntTagValue(FT_FILESIZE);
		case 3: //sources asc
			return item2->GetIntTagValue(FT_SOURCES) - item1->GetIntTagValue(FT_SOURCES);
		case 13: //sources desc
			return item1->GetIntTagValue(FT_SOURCES) - item2->GetIntTagValue(FT_SOURCES);

		case 4: //availabl asc
			return 0;
		case 14: //sources desc
			return 0;

		case 5: //type asc
			return GetFiletypeByName(item1->GetFileName()).Compare(GetFiletypeByName(item2->GetFileName()));

		case 15: //type  desc
			return GetFiletypeByName(item2->GetFileName()).Compare(GetFiletypeByName(item1->GetFileName()));

		case 6: //filahash asc
			return memcmp(item1->GetFileHash(),item2->GetFileHash(),16);
		case 16: //filehash desc
			return memcmp(item2->GetFileHash(),item1->GetFileHash(),16);
		default:
			return 0;
	}
}

void CSearchListCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{
	if (GetSelectionMark() != (-1))  m_SearchFileMenu.TrackPopupMenu(TPM_LEFTALIGN |TPM_RIGHTBUTTON,point.x,point.y,this);
}



BOOL CSearchListCtrl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	CSearchFile* file ;
	int item;

	if (GetSelectionMark() != (-1)){
		file = (CSearchFile*)GetItemData(GetSelectionMark());
		switch (wParam){
			case MP_GETED2KLINK:
				{
					theApp.CopyTextToClipboard(theApp.CreateED2kLink(file));
					break;
				}
			case MP_GETHTMLED2KLINK:
				{
					theApp.CopyTextToClipboard(theApp.CreateHTMLED2kLink(file));
					break;
				}
			case MP_RESUME:
				{
					theApp.emuledlg->searchwnd.OnBnClickedSdownload();
					break;
				}
			case MP_REMOVE:
				{
					if( this->GetSelectedCount() == 1 )
						theApp.searchlist->RemoveResults(file->GetSearchString());
					else if(this->GetSelectedCount() > 1 ){
                        ShowWindow(SW_HIDE); 
						while (GetSelectionMark() != (-1)) //(pos != NULL) 
						{
							file = (CSearchFile*)GetItemData(GetSelectionMark());
							theApp.searchlist->RemoveResults( file->GetSearchString());
						}
						ShowWindow(SW_SHOW);
					}
					break;
				}
			case MP_REMOVEALL:
				{
					ClearAll();
					break;
				}
			case MP_REMOVESELECTED:
				{
					ShowWindow(SW_HIDE); 
					POSITION pos;
					while (GetFirstSelectedItemPosition()!=NULL) 
					{
						pos=GetFirstSelectedItemPosition();
						item = this->GetNextSelectedItem(pos); 
						theApp.searchlist->RemoveResults( (CSearchFile*)this->GetItemData(item) );
					}
					ShowWindow(SW_SHOW);
					break;
				}
		}
	}
	return true;
}

void CSearchListCtrl::ClearAll(){
	theApp.searchlist->Clear();
	DeleteAllItems();
}
